package asd.td8;

import asd.graphe.*;

/**
 * EXERCICE 12 - TD 8
 * Une classe pour la recherche des points d'articulation.
 */
public class Articulation {

    private GrapheNonOriente G;
    
    private int[] d; // la date de dbut de traitement des sommets
    private int[] m; // l'attribut m 
    private int[] p; // p[j] = i <==> i est le pre de j
    private int[] c; // la couleur des sommets
    private int t;   // le temps

    private static final int blanc = 0;
    private static final int gris  = 1;
    private static final int noir  = 2;

    /**
     * Construit un objet Articulation pour dterminer les points
     * d'articulation du graphe G.
     * Prcondition : G est connexe.
     * Complexit : O(1).
     */
    public Articulation(GrapheNonOriente G) {
	this.G = G;
	this.d = new int[G.ordre()];
	this.m = new int[G.ordre()];
	this.p = new int[G.ordre()];
	this.c = new int[G.ordre()];
    }

    /**
     * Retourne un tableau de boolean pda tel que pour tout sommet
     * u de G, pda[u] = true <==> u est un point d'articulation de G.
     * Complexit : O(A) o A est le nombre d'artes de G.
     */
    public void parcourir() {

	t = 1;       // on initialise le temps courant
        int ndf = 0; // le nombre de fils du sommet 0, racine du parcours
        boolean[] pda = new boolean[G.ordre()]; // pda[u] <=> u est un 
                                                // point d'articulation
	for ( int i = 0; i < G.ordre(); i++ ) {
	    c[i] = blanc;
	    p[i] = -1;
            pda[i] = false;
	}
        // tous les sommets sont blancs, aucun sommet n'a de pre et
        // aucun sommet n'est un point d'articulation

	c[0] = gris; // on dcouvre 0
	d[0] = t++;  //  la date d[0] = 1
        m[0] = d[0]; // et on positionne m[0] = 1

	Adjacents adj = G.adjacents(0);
	
	while ( adj.nonVide() ) {
	    int s = adj.suivant();

	    if ( c[s] == blanc ) { // s devient un fils de 0
                ndf++;             // 0 a un fils de plus
		p[s] = 0;          // 0 est le pre de s
		visiter(s,pda);        // on visite s en profondeur
	    }
	}
	c[0] = noir; // on termine 0

        if ( ndf > 1 ) {   // 0 a au moins 2 fils dans l'arbre en 
            articulation(0);
        }
        for ( int i = 1; i <  G.ordre(); i++ ) {
            if ( pda[i] ) { // i est un point d'articulation
                articulation(i);
            }
        }
    }

    /**
     * Visite le graphe en profondeur  partir du sommet i, pour
     * tout u un sommet visit, calcule m[u] et positionne pda[u] 
     * Prcondition : i est blanc
     * Postrelation : i est noir et pour tous les sommets u visits
     * m[u] a t calcul
     * Complexit : O(Ai) o Ai est le nombre d'artes visites
     */
    private void visiter(int i, boolean[] pda) {

	c[i] = gris; // on dcouvre i
	d[i] = t++;  //  la date d[i] = t (le temps courant)
        m[i] = d[i]; // d[i] est la valeur maximale de m[i]

	Adjacents adj = G.adjacents(i);
	
	while ( adj.nonVide() ) {
	    int s = adj.suivant();

	    if ( c[s] == blanc ) {
		p[s] = i;             // le pre de s est i
		visiter(s,pda);       // on visite s en profondeur
                if ( m[s] >= d[i] ) { // si s ne remonte pas strictement,
                    pda[i] = true;    // i est un point d'articulation
                }
                if ( m[s] < m[i] ) {
                    m[i] = m[s];      // on met  jour m[i]
                }
	    }
            else {
                if ( c[s] == gris && s != p[i] ) { // (s,i) est une arte
                    if ( d[s] < m[i] ) {           // retour
                        m[i] = d[s];  // on met  jour m[i]
                    }
                }
            }
	}
	c[i] = noir; // on termine i
    }

    // Pour afficher les points d'articulation
    //
    private void articulation(int i) {
        System.out.println(i+ " est un point d'articulation");
    }
}
