package asd.td8;

import asd.graphe.Adjacents;
import asd.graphe.GrapheNonOriente;

/**
 * EXERCICE 6 - TD 8
 * Une classe pour tester si un graphe non orient est biparti.
 */
public class Biparti {

    private GrapheNonOriente G;
    
    private int[] c; // la couleur des sommets

    private static final int vert  = -1; // vert et rouge sont les deux
    private static final int rouge =  1; // couleurs complmentaires
    private static final int blanc =  0; // un sommet est blanc s'il n'est pas
                                         // encore color
    /**
     * Construit un objet Biparti pour tester si le graphe G est biparti.
     */
    public Biparti(GrapheNonOriente G) {
	this.G = G;
	this.c = new int[G.ordre()];
    }

    /**
     * Teste si le graphe est biparti.
     * Complexit : O(S + A) o S est l'ordre du graphe et A le nombre d'arcs.
     */
    public boolean biparti() { // parcourir
        boolean biparti = true;
        for ( int i = 0; i < G.ordre(); i++ ) {
            c[i] = blanc;
        }
        for ( int i = 0; i < G.ordre() && biparti; i++ ) {
            if ( c[i] == blanc ) {
                biparti = biparti(i,vert);
            }
        }
        // INVARIANT :
        // toutes les composantes connexes du sous-graphe engendr par les
        // sommets 0..i-1 sont biparties
        //
        return biparti;
    }

    /**
     * Teste si le sous-graphe de racine i est biparti.
     * Prcondition : i est blanc.
     * Postrelation : i a la couleur couleur, si le sous-graphe de racine i est
     * biparti.
     * Complexit : O(Ai) o Ai est le nombre d'arcs accessibles depuis i.
     */
    private boolean biparti(int i, int couleur) { // visiter
        boolean biparti = true;
        c[i] = couleur;
        Adjacents adj = G.adjacents(i);

        while ( adj.nonVide() && biparti ) {
            int s = adj.suivant();

            if ( c[s] == blanc ) { // j'explore s en profondeur et s a la
                biparti = biparti(s, autre(couleur)); // couleur inverse 
            }                                         // celle de i
            else {
                if ( c[s] == couleur ) { // s est dj visit et il a la
                    biparti = false;     // mme couleur que i
                }
            }
        }
        return biparti;
    }

    /**
     * Retourne la couleur complmentaire de k
     */
    private int autre(int k) {
        return - k;
    }
}
