package asd.td8;

import asd.graphe.Adjacents;
import asd.graphe.GrapheNonOriente;
import asd.base.FileStatique;

/**
 * EXERCICE 11 - TD 8
 * Une classe pour tester si un graphe non orient est biparti avec un
 * algorithme bas sur le parcours en largeur.
 */
public class BipartiLargeur {

    private GrapheNonOriente G; // le graphe
    private int[] c;            // c[i] = la couleur du sommet i

    private static final int vert  = -1; // vert et rouge sont les deux
    private static final int rouge =  1; // couleurs complmentaires
    private static final int blanc =  0; // un sommet est blanc s'il n'est pas
                                         // encore color

    /**
     * Construit un objet BipartiLargeur pour tester si le graphe G
     * est biparti.
     */
    public BipartiLargeur(GrapheNonOriente G) {
	this.G = G;
	this.c = new int[G.ordre()];
    }

    /**
     * Parcours chaque composante connexe du graphe G en largeur  partir d'un
     * sommet quelconque et teste si chacune de ces composantes connexes est
     * biparti.
     * Complexit : O(S + A) o S et A sont respectivement le nombre de
     * sommets et d'artes de G.
     */
    public boolean biparti() {
	for ( int i = 0; i < G.ordre(); i++ ) {
		c[i] = blanc;
	}
        for ( int i = 0; i < G.ordre(); i++ ) {
            if ( c[i] == blanc ) {
                if ( ! CCbiparti(i) ) {
                    return false;
                }
            }
        }
        return true;
    }

    /**
     * Teste si la composante connexe du graphe qui contient le sommet
     * s est biparti.
     */
    private boolean CCbiparti(int s) { // parcourir
	FileStatique f = new FileStatique(G.ordre());
	c[s] = vert;
	ajouter(s,f);

        // Invariant et variant s'entendent dans la composante connexe de s
        //
        // INVARIANT :
        // - u n'est pas blanc <==> u est dans f
	// - le sous-graphe engendr par SNB est biparti (SNB = l'ensemble
        //   des sommets non blancs)
        // VARIANT :
        // le couple (nB, |f|) diminue strictement (ordre lexicographique)
        // o nB est le nombre de sommets blancs et |f| la longueur de f
        //
	while ( nonVide(f) ) {
	    int u = enlever(f);
	    Adjacents adj = G.adjacents(u);

	    while ( adj.nonVide() ) {
		int v = adj.suivant();
                if ( c[v] == blanc ) {
                    c[v] = autre(c[u]);
		    ajouter(v,f);
		}
                else {
                    if ( c[v] == c[u] ) {
                        return false;
                    }
                }
	    }
	}
        return true;
    }

    /**
     * Retourne la couleur complmentaire de k
     */
    private int autre(int k) {
        return - k;
    }

    // pour se simplifier la vie

    private boolean nonVide(FileStatique f) {
        return ! f.vide();
    }

    private void ajouter(int i, FileStatique f) {
        f.ajouter(new Integer(i));
    }

    private int enlever(FileStatique f) {
        int i = ((Integer) f.suivant()).intValue();
        f.retirer();
        return i;
    }
}
