package asd.td8;

import asd.graphe.Adjacents;
import asd.graphe.GrapheNonOriente;

/**
 * EXERCICE 3 - TD 8
 * Une classe pour tester si un graphe non orient est acyclique.
 */
public class CycleNonOriente {

    private GrapheNonOriente G;
    private boolean[] visite;
    private int[] pere;

    /**
     * Construit un objet CycleNonOriente pour tester si le graphe G 
     * est acyclique.
     */
    public CycleNonOriente(GrapheNonOriente G) {
        this.G = G;
        this.visite = new boolean[G.ordre()];
        this.pere = new int[G.ordre()];
    }

    /**
     * Teste si le graphe est acyclique.
     */
    public boolean acyclique() {
        for ( int i = 0; i < G.ordre(); i++ ) {
            visite[i] = false; // aucun sommet n'a t visit
            pere[i] = -1;      // aucun sommet n'a de pre
        }
        for ( int i = 0; i < G.ordre(); i++ ) {
            if ( ! visite[i] ) {
                if ( ! acyclique(i) ) {
                    return false;
                }
            }
        }
        return true;
    }

    /**
     * Visite le sous-graphe de sommet i en profondeur et recherche un cycle.
     * Prcondition : i n'est pas visit
     * Postrelation : i est visit et tous les descendants de i ont t
     * visits.
     * Complexit : O(Ai) o Ai est le nombre d'arcs accessibles depuis i.
     */
    private boolean acyclique(int i) {
        visite[i] = true;
        Adjacents adj = G.adjacents(i);

        while ( adj.nonVide() ) {
            int s = adj.suivant();
            if ( ! visite[s] ) {
                pere[s] = i;
                if ( ! acyclique(s) )
                    return false;
            }
            else
                if ( s != pere[i] )
                    return false;
        }
        return true;
    }
}
