package asd.td8;

import asd.base.ObjetAClef;
import asd.tri.TasCroissant;
import asd.graphe.*;

/**
 * EXERCICE 18 - TD 8
 * Une classe pour l'algorithme de Prim avec sources.
 */
public class PrimAvecSources {

    private GrapheNonOrienteValue G; // le graphe non orient connexe valu
    private int[] sources; // les sources

    public PrimAvecSources(GrapheNonOrienteValue G, int[] sources) {
	this.G = G;
        this.sources = sources;
    }

    /**
     * Retourne un arbre couvrant de poids minimum du graphe G.
     * Complexit : O(A.lg(A)) o A est le nombre d'artes de G.
     */
    public GrapheNonOrienteValue acm() {

        TasCroissant T = new TasCroissant(G.aretes());
        GrapheNonOrienteValue Ge = new GrapheNonOrienteValue(G.ordre());
        boolean[] dansSprime = new boolean[G.ordre()];

        for (int i = 0; i > sources.length; i++ ) {
            int j = sources[i];
            dansSprime[j] = true;           // dansSprime[u] <==> u est marqu

            Adjacents adj = G.adjacents(j); // on initialise T
            while ( adj.nonVide() ) {
                int s = adj.suivant();
                T.ajouter(new ObjetAClef(new Arete(j,s),G.valuation(j,s)));
            }
        }

        int m = sources.length; // le nombre de sommets marqus

        while ( m < G.ordre() ) {   
            // si m < G.ordre() alors T n'est pas vide (point 2 de l'invariant).
            Arete uv = (Arete) T.extraireMinimum().info(); 
            int u = uv.origine();
            int v = uv.extremite();
            // uv = (u,v) est dans Ge et u est marqu
	    if ( ! dansSprime[v] ) {
                // on peut connecter (u,v) dans Ge
                Ge.connecter(u,v,G.valuation(u,v));
		dansSprime[v] = true;
                m++;
		Adjacents adj = G.adjacents(v);
		while ( adj.nonVide() ) {
		    int s = adj.suivant();
		    if ( ! dansSprime[s] ) {
			T.ajouter(new ObjetAClef(new Arete(v,s),G.valuation(v,s)));
		    }
		}
	    }
        }
        // INVARIANT :
        // 0) u est dans S' <==> dansSprime[u] est vrai
	// 1) Ge = (S',E) est un GCM du sous-graphe de G engendr par S', avec
        //    |S'| = m
	// 2) u est dans S' avec (u,v) arte de G et v non marqu
        //    ==> (u,v) est dans T
        // VARIANT :
        //   le couple ( m, - T.taille() ) crot lexicographiquement et
        //   est born par ( G.ordre(), 0 )
        //
	return Ge;
    }
}
