package asd.td8;

import asd.base.ObjetAClef;
import asd.tri.TasCroissant;
import asd.graphe.*;

/**
 * EXERCICE 14 - TD 8
 * Une classe pour l'algorithme de Prim, pour des graphes non orients
 * ventuellement non connexes, valus.
 */
public class PrimNonConnexe {

    private GrapheNonOrienteValue G; // le graphe non orient connexe valu

    public PrimNonConnexe(GrapheNonOrienteValue G) {
	this.G = G;
    }

    /**
     * Retourne une foret couvrante de poids minimum du graphe G.
     * Complexit : O(A.lg(A)) o A est le nombre d'artes de G.
     */
    public GrapheNonOrienteValue fcm() {

        TasCroissant T = new TasCroissant(G.aretes());
        GrapheNonOrienteValue FCM = new GrapheNonOrienteValue(G.ordre());
        boolean[] marque = new boolean[G.ordre()];

        int ndsm = 0; // le nombre de sommets marqus
        for ( int i = 0; i < G.ordre(); i++ ) { // on fait Prim sur les comp. connexes
            if ( ! marque[i] ) {
                marque[i] = true; // marque[u] <==> u est marqu
                ndsm++;

                Adjacents adj = G.adjacents(i); // on initialise T
                while ( adj.nonVide() ) {
                    int s = adj.suivant();
                    T.ajouter(new ObjetAClef(new Arete(i,s),G.valuation(i,s)));
                }

                while ( T.taille() > 0 ) {
                    // si ndsm < G.ordre() alors T n'est pas vide (point 4 de l'invariant).
                    Arete uv = (Arete) T.extraireMinimum().info(); 
                    int u = uv.origine();
                    int v = uv.extremite();
                    // uv = (u,v) est dans FCM, u est marqu et connect dans FCM
                    if ( ! marque[v] ) {
                        // on peut connecter (u,v) dans FCM
                        FCM.connecter(u,v,G.valuation(u,v));
                        marque[v] = true;
                        ndsm++;
                        adj = G.adjacents(v);
                        while ( adj.nonVide() ) {
                            int s = adj.suivant();
                            if ( ! marque[s] ) {
                                T.ajouter(new ObjetAClef(new Arete(v,s),G.valuation(v,s)));
                            }
                        }
                    }
                }
	    }
        }
        // INVARIANT :
	// 1) E, l'ensemble des artes de FCM, est un sous-FCM de G
        // 2) S', l'ensemble des sommets marqus, est une coupure qui respecte E
        // 3) T = (S',E) est un arbre
	// 4) u est marqu (i.e. dans S') avec (u,v) arte de G et v non marqu
        //    ==> (u,v) est dans T
        // VARIANT :
        //   le couple ( ndsm, - T.taille() ) crot lexicographiquement et
        //   est born par ( G.ordre(), 0 )
        //
	return FCM;
    }
}
