package asd.td8;

import asd.graphe.Adjacents;
import asd.graphe.GrapheOriente;

/**
 * EXERCICE 7 - TD 8
 * Une classe pour dterminer si un graphe orient possde une racine.
 */
public class Racine {

    private GrapheOriente G;
    
    private int[] c; // la couleur des sommets

    private static final int blanc = 0;
    private static final int gris  = 1;
    private static final int noir  = 2;

    /**
     * Construit un objet Racine pour la dterminer si le graphe possde
     * une racine.
     */
    public Racine(GrapheOriente G) {
	this.G = G;
	this.c = new int[G.ordre()];
    }

    /**
     * Teste si le graphe admet une racine.
     * Valeur rendue : i, tel que si i != -1, i est un sommet racine du graphe,
     * et si i = -1, le graphe ne possde pas de racine.
     * Complexit : O(S + A) o S est l'ordre du graphe et A le nombre d'arcs.
     */
    public int racine() {
        int racine = 0; // la racine potentielle
	
	for ( int i = 0; i < G.ordre(); i++ ) {
	    c[i] = blanc;
	}
        // tous les sommets sont blancs
	for ( int i = 0; i < G.ordre(); i++ ) {
	    if ( c[i] == blanc ) {
                racine = i;
		visiter(i);
	    }
	}
        // racine est le sommet tel que f[racine] > f[u], pour tout sommet u
        if ( racine == 0 ) { // on a atteint tous les sommets depuis 0
            return 0;        // ==> 0 est une racine du graphe
        }
        // on repeint tous les sommets en blanc pour le prochain parcours
	for ( int i = 0; i < G.ordre(); i++ ) {
	    c[i] = blanc;
	}
        // tous les sommets sont  nouveau blancs
        if ( visiter(racine) == G.ordre() ) {
            return racine;
        }
        return -1;
    }

    /**
     * Visite le sous-graphe de sommet i en profondeur et retourne le nombre
     * de sommets visits.
     * Prcondition : i est blanc.
     * Postrelation : i est noir et tous les descendants de i ont t
     * visits.
     * Valeur rendue : le nombre de sommets visits (i inclus).
     * Complexit : O(Ai) o Ai est le nombre d'arcs accessibles depuis i.
     */
    private int visiter(int i) {
	c[i] = gris;
	Adjacents adj = G.adjacents(i);
        int ndsv = 1; // nombre de sommets visits
	
	while ( adj.nonVide() ) {
	    int s = adj.suivant();

            // dbut de tratement
            if ( c[s] == blanc ) {
		ndsv += visiter(s);
            }
            // fin de tratement
	}
	c[i] = noir;
        return ndsv;
    }
}
