package asd.td8;

import asd.graphe.Adjacents;
import asd.graphe.GrapheOriente;
import asd.base.PileStatique;

/**
 * EXERCICE 13 - TD 8
 * Une classe pour dterminer si un graphe orient est semi-connexe.
 */
public class SemiConnexe {

    private GrapheOriente G;  // le graphe orient
    private GrapheOriente TG; // TG = transpos(G)
    
    private int[] c; // la couleur des sommets

    private static final int blanc = 0;
    private static final int gris  = 1;
    private static final int noir  = 2;

    /**
     * Cre un objet SemiConnexe pour dterminer si un G est semi-connexe.
     */
     public SemiConnexe(GrapheOriente G) {
        this.G = G;
	this.TG = new GrapheOriente(G.ordre());
        this.c = new int[G.ordre()];
     }

    /**
     * Teste si le graphe est semi-connexe.
     * Complexit : O(S + A) o S et A sont respectivement le nombre de
     * sommets et d'arcs du graphe.
     */
    public boolean semiConnexe() {
	return semiConnexe(topo());
    }

    /**
     * Ralise le parcours en profondeur du graphe TG, en examinant les
     * sommets dans l'ordre o ils sont enlevs de la pile (l'ordre
     * topologique calcul sur le graphe G), numrote les arbres de la fort
     * en profondeur correspondante, et teste la proprit de la question 1
     * de l'exercie 11.
     * Complexit : O(S + A) o S et A sont respectivement le nombre de
     * sommets et d'arcs du graphe TG (et du graphe G).
     */
    private boolean semiConnexe(PileStatique pile) {
	int n = 1;                      // le numro de la CFC courante
        int[] cfc = new int[G.ordre()]; // cfc[i] = k <==> le sommet i est
                                        // dans la CFC numro k
        
        for ( int i = 0; i < G.ordre(); i++ ) {
            c[i] = blanc;
        }

        boolean ok = true; // ok <=> il exite un arc (couvrant) allant de la
                           // CFC numro n vers la CFC numro n - 1
        visiter(depiler(pile),n++,cfc); // sauf pour la CFC numro 1
        
	while ( ok && nonVide(pile) ) {
            int i = depiler(pile);
            if ( c[i] == blanc ) {
                ok = visiter(i,n++,cfc);
            }
        }
        return ok;
    }
    
    /**
     * Visite le sommet i et ses descendants dans le graphe TG, attribue
     *  tous ces sommets la CFC numro n et teste s'il exite un arc
     * (couvrant) i --> u tel que cfc[i] = n et cfc[u] = n - 1.
     * Prcondition : i est blanc.
     * Postrelation : i est pas blanc et le numro de CFC n a t attribu
     *  tous les sommets accessibles depuis i (i inclus).
     * Complexit : O(Ai) o Ai est le nombre d'arcs accessibles depuis i.
     */
    private boolean visiter(int i, int n, int[] cfc) {
        c[i] = gris;        // i vient d'tre dcouvert
	cfc[i] = n;         // i appartient  la CFC courante (numro n)
        boolean ok = false; // ok <=> il existe un arc (couvrant) i --> u
                            // tel que cfc[i] = n et cfc[u] = n - 1
        Adjacents adj = TG.adjacents(i);
        
        while ( adj.nonVide() ) {
            int s = adj.suivant();
            if ( c[s] == blanc ) {
                if ( visiter(s,n,cfc) ) {
                    ok = true;
                }
            }
            else {
                if ( c[s] == noir && cfc[s] == n - 1 ) {
                    ok = true;
                }
            }
        }
        c[i] = noir;
        return ok;
    }

    /**
     * Trie le graphe G topologiquement et retourne une pile contenant tous les
     * sommets du graphe, telle que pour tout sommet u et v dans la pile, si
     * u est dpil avant v alors soit u et v ne sont pas connects, soit il
     * existe dans G un chemin de u  v. La mthode topo initialise galement
     * le graphe transpos TG.
     * Complexit : O(S + A) o S et A sont respectivement le nombre de
     * sommets et d'arcs du graphe G (et du graphe TG).
     */
    private PileStatique topo() {
	PileStatique pile = new PileStatique();

        for ( int i = 0; i < G.ordre(); i++ ) {
            if ( c[i] == blanc ) {
                visiter(i,pile);
            }
        }
	return pile;
    }

    /**
     * Visite le sous-graphe de G de sommet i en profondeur et empile tous les
     * sommets du sous-graphe de racine i dans la pile, dans l'ordre inverse
     * de l'ordre topologique.
     * Prcondition : i est blanc.
     * Postrelation : i n'est pas blanc et tous les sommets accessibles depuis
     * i (i inclus) ont t empil dans la pile, dans l'ordre inverse de
     * l'ordre topologique.
     * Complexit : O(Ai) o Ai est le nombre d'arcs accessibles depuis i.
     */
     private void visiter(int i, PileStatique pile) {
        c[i] = gris;

        Adjacents adj = G.adjacents(i);
        while ( adj.nonVide() ) {
            int s = adj.suivant();
	    TG.connecter(s,i);     // on construit TG
            if ( c[s] == blanc ) {
                visiter(s,pile);   // visite "en profondeur"
            }
        }
        // ( c[i] = noir ) fin de traitement de i
	empiler(i,pile);
    }

    private boolean nonVide(PileStatique f) {
        return ! f.vide();
    }

    private void empiler(int i, PileStatique f) {
        f.empiler(new Integer(i));
    }

    private int depiler(PileStatique f) {
        int i = ((Integer) f.sommet()).intValue();
        f.depiler();
        return i;
    }
}
